@@ -109,7 +109,7 @@ def namedlock(name):
     return namedlock.locks[name]
 
 
-def httpdo(url, method="GET", data=None) -> attrdict.AttrDict:
+def httpdo(url, method="GET", data=None, timeout=None) -> attrdict.AttrDict:
     """
     thread safe http request
 
@@ -118,7 +118,7 @@ def httpdo(url, method="GET", data=None) -> attrdict.AttrDict:
     """
     p = urlparse(url)
     with namedlock(p.scheme + "://" + p.netloc):
-        return _unsafe_httpdo(url, method, data)
+        return _unsafe_httpdo(url, method, data, timeout)
 
 
 @functools.lru_cache(1024)
@@ -129,7 +129,7 @@ def _requests_session_pool_get(scheme, netloc):
 def _is_tmq_platform() -> bool:
     return os.getenv("TMQ") == "true"
 
-def _unsafe_httpdo(url, method='GET', data=None):
+def _unsafe_httpdo(url, method='GET', data=None, timeout=None):
     """
     Do HTTP Request
     """
@@ -139,13 +139,15 @@ def _unsafe_httpdo(url, method='GET', data=None):
         print("Shell$ curl -X {method} -d '{body}' '{url}'".format(
             method=method.upper(), body=body or '', url=url))
 
+    if timeout is None:
+        timeout = HTTP_TIMEOUT
     try:
         u = urlparse(url)
         request_session = _requests_session_pool_get(u.scheme, u.netloc)
         response = request_session.request(method,
                                            url,
                                            json=data,
-                                           timeout=HTTP_TIMEOUT)
+                                           timeout=timeout)
     except (requests.ConnectionError, requests.ReadTimeout) as e:
         raise
 
@@ -308,9 +310,9 @@ class BaseClient(object):
 
     def is_ready(self) -> bool:
         try:
-            self.status()
+            self.http.get("status", timeout=3)
             return True
-        except:
+        except Exception as e:
             return False
 
     def wait_ready(self, timeout=120, noprint=False) -> bool:
@@ -387,7 +389,8 @@ class BaseClient(object):
                method: str,
                urlpath: str,
                data: Optional[dict] = None,
-               with_session: bool = False) -> attrdict.AttrDict:
+               with_session: bool = False,
+               timeout: Optional[float] = None) -> attrdict.AttrDict:
         """ do http request """
         urlpath = "/" + urlpath.lstrip("/")  # urlpath always startswith /
 
@@ -412,7 +415,7 @@ class BaseClient(object):
                 url = urljoin(self.__wda_url, "session", self.session_id,
                               urlpath)
             run_callback(Callback.HTTP_REQUEST_BEFORE)
-            response = httpdo(url, method, data)
+            response = httpdo(url, method, data, timeout)
             run_callback(Callback.HTTP_REQUEST_AFTER, response=response)
             return response
         except Exception as err:
@@ -452,7 +455,7 @@ class BaseClient(object):
         """Hit healthcheck"""
         return self.http.get('/wda/healthcheck')
 
-    def locked(self):
+    def locked(self) -> bool:
         """ returns locked status, true or false """
         return self.http.get("/wda/locked").value
 
@@ -596,6 +599,10 @@ class BaseClient(object):
                                                     {}),  # 兼容旧版的wda
         }
 
+        # when device is Locked, it is unable to start app
+        if self.locked():
+            self.unlock()
+
         try:
             res = self.http.post('session', payload)
         except WDAEmptyResponseError:
@@ -959,11 +966,10 @@ class BaseClient(object):
             namedtuple: eg
                 Size(width=320, height=568)
         """
-        w, h = self._unsafe_window_size()
-        if w == 0 and h == 0:
-            # FIXME(ssx): launch another app is not a good idea, but I did'nt found some other way
-            self.unlock()
-            self.session("com.apple.Preferences")
+        size = self._unsafe_window_size()
+        if min(size) > 0:
+            return size
+        _ = self.orientation # after this operation, will safe to get window_size
         return self._unsafe_window_size()
 
     def _unsafe_window_size(self):
@@ -1102,7 +1108,7 @@ class Alert(object):
     def watch_and_click(self,
                         buttons: Optional[list] = [
                             "使用App时允许", "无线局域网与蜂窝网络", "好", "稍后", "稍后提醒", "确定",
-                            "允许", "以后", "打开", "录屏"
+                            "允许", "以后", "打开", "录屏", "Allow", "OK", "YES", "Yes", "Later", "Close"
                         ],
                         interval: float = 2.0):
         """ watch and click button
@@ -1307,10 +1313,9 @@ class Selector(object):
         if self._name:
             qs.append("name == '%s'" % self._name)
         if self._name_part:
-            qs.append("name CONTAINS '%s'" % self._name_part)
+            qs.append("name CONTAINS %r" % self._name_part)
         if self._name_regex:
-            qs.append("name MATCHES '%s'" %
-                      self._name_regex.encode('unicode_escape'))
+            qs.append("name MATCHES %r" % self._name_regex)
         if self._label:
             qs.append("label == '%s'" % self._label)
         if self._label_part:
@@ -1520,7 +1525,7 @@ class Element(object):
     @property
     def info(self):
         return {
-            "id": self.session_id,
+            "id": self._id,
             "label": self.label,
             "value": self.value,
             "text": self.text,
@@ -1670,8 +1675,10 @@ class Element(object):
 
 class USBClient(Client):
     """ connect device through unix:/var/run/usbmuxd """
-    def __init__(self, udid: str = "", port: int = 8100):
+    def __init__(self, udid: str = "", port: int = 8100, wda_bundle_id=None):
         super().__init__(url="usbmux://{}:{}".format(udid, port))
+        self.__udid = udid
+        self.__wda_bundle_id = wda_bundle_id
 
         if not self.is_ready():
             self._start_xctest()
@@ -1684,7 +1691,14 @@ class USBClient(Client):
             return False
 
         logger.info("WDA is not running, exec: tins xctest")
-        p = subprocess.Popen([tins_path, 'xctest'])
+        args = []
+        if self.__udid:
+            args.extend(['-u', self.__udid])
+        args.append('xctest')
+        if self.__wda_bundle_id:
+            args.extend(['-B', self.__wda_bundle_id])
+
+        p = subprocess.Popen([tins_path] + args)
         time.sleep(3)
         assert p.poll() is None
         return self.wait_ready()
